
import json

from urllib.parse import urlencode
from .base import BaseShortener, ShortenerServiceError


BITLY_API_VERSION = '2.0.1'
BITLY_SERVICE_URL = 'http://api.bit.ly/'


class BitlyError(ShortenerServiceError):
    pass


class Bitly(BaseShortener):

    exception_class = BitlyError
    service_url = BITLY_SERVICE_URL

    def __init__(self, login, api_key, logger=None):
        super().__init__(api_key, logger=logger)
        self.login = login
        self.default_request_params = {
            'version': BITLY_API_VERSION,
            'format': 'json',
            'login':  self.login,
            'apiKey': self.api_key,
        }

    def _get_request_url(self, action, param_key, param_value):
        request_params = self.default_request_params
        request_params[param_key] = param_value
        request_params = request_params.items()

        encoded_params = urlencode(request_params)
        return "%s%s?%s" % (self.service_url, action, encoded_params)

    def _is_response_success(self, response):  # pylint: disable=no-self-use
        return response.get('statusCode') == 'OK'

    def _get_error_from_response(self, response):  # pylint: disable=no-self-use
        error_code = response.get('errorCode')
        error_message = response.get('errorMessage')

        error = 'Invalid Response. '
        if error_code:
            error = '%s: ' %error_code
        if error_message:
            error += error_message

        return error

    def shorten_url(self, long_url):
        request_url = self._get_request_url('shorten', 'longUrl', long_url)
        headers, response = self._do_http_request(request_url)  # pylint: disable=unused-variable

        response = json.loads(response)
        if not self._is_response_success(response):
            msg = self._get_error_from_response(response)
            raise self.exception_class(msg)

        results_dict = response.get('results')
        result = results_dict.get(long_url)
        return result.get('shortUrl')

    def expand_url(self, short_url):
        request_url = self._get_request_url('expand', 'shortUrl', short_url)
        headers, response = self._do_http_request(request_url)  # pylint: disable=unused-variable

        response = json.loads(response)
        if not self._is_response_success(response):
            msg = self._get_error_from_response(response)
            raise self.exception_class(msg)

        results_dict = response.get('results')
        return results_dict.values()[0].get('longUrl')


    def get_short_url_info(self, short_url):
        request_url = self._get_request_url('info', 'shortUrl', short_url)
        headers, response = self._do_http_request(request_url)  # pylint: disable=unused-variable

        response = json.loads(response)
        if not self._is_response_success(response):
            msg = self._get_error_from_response(response)
            raise self.exception_class(msg)

        results_dict = response.get('results')
        return results_dict.values()[0]

    def get_stats(self, short_url):
        request_url = self._get_request_url('stats', 'shortUrl', short_url)
        headers, response = self._do_http_request(request_url)  # pylint: disable=unused-variable

        response = json.loads(response)
        if not self._is_response_success(response):
            msg = self._get_error_from_response(response)
            raise self.exception_class(msg)

        results_dict = response.get('results')
        return results_dict




import logging
import requests

from pyshorturl.conf import USER_AGENT_STRING


LOGGER = logging.getLogger('pyshorturl.shortener')


class ShortenerServiceError(Exception):
    pass


class BaseShortener():
    """Base class for the url shorteners in the lib"""

    exception_class = ShortenerServiceError
    service_url = None

    def __init__(self, api_key, *, logger=None):
        self.headers = {
            'User-Agent': USER_AGENT_STRING,
        }
        self.api_key = api_key
        self.logger = logger if logger else LOGGER

    def _do_http_request(self, request_url, *, json=None, data=None, headers=None):

        if not self.service_url:
            raise self.exception_class('Service URL is empty. Cannot proceed with URL shortening.')

        if headers:
            self.headers.update(headers)

        try:
            if data or json:
                self.logger.debug('Sending POST request to %s' % request_url)
                self.logger.debug('Headers: %s' % self.headers)
                self.logger.debug('Data: %s' % data)
                self.logger.debug('JSON: %s' % json)
                response = requests.post(request_url, json=json, data=data, headers=self.headers)

            else:
                self.logger.debug('Sending GET request to %s' % request_url)
                self.logger.debug('Headers: %s' % self.headers)
                response = requests.get(request_url, headers=self.headers)

            response.raise_for_status()
            if response.headers.get('Content-Type') == 'application/json':
                return (response.headers, response.json())
            return (response.headers, response.content)

        except requests.exceptions.HTTPError as e:  # pylint: disable=invalid-name
            raise self.exception_class(e)

    def set_api_key(self, api_key):
        self.api_key = api_key

    def shorten_url(self, long_url):
        raise NotImplementedError()

    def expand_url(self, short_url):  # pylint: disable=no-self-use
        response = requests.get(short_url)
        return response.headers.get('location')

    def get_qr_code(self, short_url):
        raise NotImplementedError()

    def write_qr_image(self, short_url, image_path):
        """Obtain the QR code image corresponding to the specified `short_url`
        and write the image to `image_path`.

        If the caller does not intent to use the png image file, `get_qr_code()`
        may be used to obtain raw image data.

        Keyword arguments:
            long_url -- The url to be shortened.

        Returns:
            Returns raw png image data that constitutes the qr code image.

        Exceptions:
            `ShortenerServiceError` - In case of error
        """
        image_data = self.get_qr_code(short_url)

        # pylint: disable=invalid-name
        fd = open(image_path, 'w')
        fd.write(image_data)
        fd.close()




import json

from .base import BaseShortener, ShortenerServiceError


GOOGL_SERVICE_URL = 'https://www.googleapis.com/urlshortener/v1/url'


class GooglError(ShortenerServiceError):
    pass


class Googl(BaseShortener):

    exception_class = GooglError
    service_url = GOOGL_SERVICE_URL

    def __init__(self, api_key=None, logger=None):
        super().__init__(api_key, logger=logger)
        # goo.gl mandates that requests containing JSON content bodies must be
        # accompanied by a "Content-Type: application/json" request header.
        # Otherwise, the request will result in an Error (400: Bad Request).
        self.headers['Content-Type'] = 'application/json'

    def _get_request_url(self, url_params={}):  # pylint: disable=dangerous-default-value
        request_url = self.service_url
        param_list = []

        if self.api_key:
            param_list.append('key=%s' %self.api_key)

        for param in url_params.keys():
            param_list.append('%s=%s' %(param, url_params.get(param)))
        params = '&'.join(param_list)

        if params:
            request_url = request_url + '?' + params

        return request_url

    def shorten_url(self, long_url):
        data = """{"longUrl": "%s"}""" %long_url

        request_url = self._get_request_url()
        headers, response = self._do_http_request(request_url, data=data)  # pylint: disable=unused-variable

        response = json.loads(response)
        return response.get('id')

    def expand_url(self, short_url):
        url_params = {'shortUrl':short_url}
        request_url = self._get_request_url(url_params)

        headers, response = self._do_http_request(request_url)  # pylint: disable=unused-variable
        response = json.loads(response)

        status = response.get('status')
        if status == 'REMOVED':
            raise GooglError(
                'Unable to obtain long url for %s. The url status is REMOVED.' % short_url)
        elif status == 'OK':
            response = response.get('longUrl')

        return response

    def get_qr_code(self, short_url):
        qr_url = short_url + '.qr'
        headers, response = self._do_http_request(qr_url)  # pylint: disable=unused-variable

        return response




import requests

from .base import BaseShortener, ShortenerServiceError


GIMME_BAR_SERVICE_URL = "https://gimme.bar"


class GimmeBarError(ShortenerServiceError):
    pass


class GimmeBar(BaseShortener):

    exception_class = GimmeBarError
    service_url = GIMME_BAR_SERVICE_URL

    def __init__(self, api_key=None, logger=None):
        super().__init__(api_key=api_key, logger=logger)

    def _get_request_url(self):  # pylint: disable=no-self-use
        return '/'.join((self.service_url, 'shorten-me'))

    def shorten_url(self, long_url):
        data = {'url': long_url}
        request_url = self._get_request_url()
        headers = {
            'Authorization': 'Bearer {}'.format(self.api_key)
        }

        try:
            headers, response = self._do_http_request(request_url, json=data, headers=headers)
        except Exception as e:  # pylint: disable=invalid-name
            raise self.exception_class('Received Error from gim.ie', e)

        if not response:
            raise self.exception_class('Received empty response from gim.ie')

        self.logger.debug(response)
        return '/'.join((self.service_url, response['code']))




import json

from urllib.parse import urlencode
from .base import BaseShortener, ShortenerServiceError


VGD_SERVICE_URL = "http://v.gd/%s.php"


class VgdError(ShortenerServiceError):
    pass


class Vgd(BaseShortener):

    exception_class = VgdError
    service_url = VGD_SERVICE_URL

    def __init__(self, logger=None):
        super().__init__(None, logger=logger)
        self.default_request_params = {
            'format': 'json',
        }

    def _get_request_url(self, action, params):
        request_params = self.default_request_params
        request_params.update(params)
        request_params = request_params.items()

        encoded_params = urlencode(request_params)
        url = self.service_url %action

        return "%s?%s" % (url, encoded_params)

    def _get_error_from_response(self, response):  # pylint: disable=no-self-use
        """General syntax of the error response:

            { "errorcode": <error code>, "errormessage": "<error message>" }

        Specific error messages:
        Error code 1 - Please specify a URL to shorten.
        Error code 1 - there was a problem with the short URL provided (invalid/doesn't exist)
        Error code 2 - the requested short URL exists but has been disabled by us (perhaps due
                       to violation of our terms)
        Error code 3 - our rate limit was exceeded (your app should wait before trying again)
        Error code 4 - any other error (includes potential problems with our service such as a
                       maintenance period)
        """
        error = 'An error occured while executing this request.'
        error_code = response.get('errorcode')
        if error_code:
            error = str(error_code) + ': '
        error_message = response.get('errormessage')
        if error_message:
            error += error_message

        return error

    def shorten_url(self, long_url, logstats=False):  # pylint: disable=arguments-differ
        data = {'url': long_url}
        if logstats:
            data['logstats'] = 1
        request_url = self._get_request_url('create', data)
        headers, response = self._do_http_request(request_url)  # pylint: disable=unused-variable

        response = json.loads(response)
        short_url = response.get('shorturl')
        if not short_url:
            error = self._get_error_from_response(response)
            raise self.exception_class(error)

        return short_url

    def expand_url(self, short_url):
        data = {'shorturl': short_url}
        request_url = self._get_request_url('forward', data)
        headers, response = self._do_http_request(request_url)  # pylint: disable=unused-variable

        response = json.loads(response)
        long_url = response.get('url')
        if not long_url:
            error = self._get_error_from_response(response)
            raise self.exception_class(error)

        return long_url




VERSION = '0.9.4'

USER_AGENT_STRING = 'pyShortUrl v%s' % VERSION

SUPPORTED_SERVICES = (
    'goo.gl',
    'bit.ly',
    'git.io',
    'tinyurl.com',
    'v.gd',
    'is.gd',
)

SUPPORTED_DOMAINS = (
    'bit.ly',
    'bitly.com',
    'goo.gl',
    'git.io',
    'is.gd',
    'j.mp',
    'tinyurl.com',
    'v.gd',
)




import json

from urllib.parse import urlencode, urlparse
from .base import BaseShortener, ShortenerServiceError


BITLY_API_VERSION = 'v3'
BITLY_SERVICE_URL = 'http://api.bit.ly/%s/' %BITLY_API_VERSION


class BitlyError(ShortenerServiceError):
    pass


class Bitly(BaseShortener):

    exception_class = BitlyError
    service_url = BITLY_SERVICE_URL

    def __init__(self, login, api_key, logger=None):
        super().__init__(api_key, logger=logger)
        self.login = login
        self.default_request_params = {
            'format': 'json',
            'login':  self.login,
            'apiKey': self.api_key,
        }

    def _get_request_url(self, action, params):
        request_params = self.default_request_params
        request_params.update(params)
        request_params = request_params.items()

        encoded_params = urlencode(request_params)
        return "%s%s?%s" % (self.service_url, action, encoded_params)

    def _is_response_success(self, response):  # pylint: disable=no-self-use
        """A successful response will contain:
        a) status_code as 200
        b) status_txt as OK
        """
        return (response.get('status_code') == 200 and
                response.get('status_txt') == 'OK')

    # pylint: disable=inconsistent-return-statements,no-self-use
    def _get_error_from_response(self, response):
        """The exact nature of the error can be obtained from the following
        parameters in the response:

        a) status_code:
             403: Rate Limited.
             503: Unknown Error or Temporaty Unavailability.
             500: Any other invalid request/response.

        b) status_txt:
             MISSING_ARG_%s to denote a missing URL parameter,
             INVALID_%s to denote an invalid value in a request parameter,
        where %s is substituted with the name of the request parameter.

        Note that all valid responses in json and xml format will carry an HTTP
        Response Status Code of 200. This means that invalid, malformed or
        rate-limited json and xml requests may still return an HTTP response
        status code of 200.

        Some examples of error responses extracted from the ApiDocument:
        > json { "status_code": 403, "status_txt": "RATE_LIMIT_EXCEEDED", "data" : null }
        > json { "status_code": 500, "status_txt": "INVALID_URI", "data" : null }
        > json { "status_code": 500, "status_txt": "MISSING_ARG_LOGIN", "data" : null }
        > json { "status_code": 503, "status_txt": "UNKNOWN_ERROR", "data" : null }
        """

        if response.get('status_code') in (403, 500, 503):
            return response.get('status_txt', 'UNKNOWN_ERROR')

        if response.get('status_code') == 200:
            return ('Unknown Error occurred. This could be due to invalid,'
                    ' malformed or rate-limited json')

    def shorten_url(self, long_url, domain='bit.ly'):  # pylint: disable=arguments-differ
        params = {
            'uri':long_url,
            'domain':domain
        }

        request_url = self._get_request_url('shorten', params)
        headers, response = self._do_http_request(request_url)  # pylint: disable=unused-variable
        response = json.loads(response)

        if not self._is_response_success(response):
            msg = self._get_error_from_response(response)
            raise self.exception_class('Error occurred while shortening url %s: %s' % (long_url, msg))

        data_dict = response.get('data')
        short_url = data_dict.get('url')
        if not short_url:
            raise self.exception_class('Error occurred while shortening url %s' %long_url)
        return short_url


    def expand_url(self, short_url):
        # Extract hash from short_url
        url_path = urlparse(short_url)
        params = {
            'hash':url_path.path[1:]
        }

        request_url = self._get_request_url('expand', params)
        headers, response = self._do_http_request(request_url)  # pylint: disable=unused-variable
        response = json.loads(response)

        if not self._is_response_success(response):
            msg = self._get_error_from_response(response)
            raise self.exception_class('Error occurred while expanding url %s: %s' % (short_url, msg))

        data_dict = response.get('data')
        data_dict = data_dict.get('expand')
        long_url = data_dict[0].get('long_url')
        if not long_url:
            raise self.exception_class('Error occurred while expanding url %s' %short_url)
        return long_url

    def validate(self, login=None, api_key=None):
        if not login:
            login = self.login
        if not api_key:
            api_key = self.api_key

        params = {
            'x_login':login,
            'x_apiKey':api_key,
        }

        request_url = self._get_request_url('validate', params)
        headers, response = self._do_http_request(request_url)  # pylint: disable=unused-variable
        response = json.loads(response)

        if not self._is_response_success(response):
            msg = self._get_error_from_response(response)
            raise self.exception_class('Error occurred while validating account %s: %s' % (self.login, msg))

        data = response.get('data')
        return data.get('valid')




from .v_gd import Vgd, VgdError


ISGD_SERVICE_URL = "http://is.gd/%s.php"


class IsgdError(VgdError):
    pass


class Isgd(Vgd):

    exception_class = IsgdError
    service_url = ISGD_SERVICE_URL




""" Install script for pyShortUrl """

import os
from setuptools import setup

from pyshorturl.conf import VERSION

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "pyShortUrl",
    version = VERSION,
    author = "Fictive Kin (mostly of previous work by Parth Bhatt)",
    author_email = "systems@fictivekin.com",
    description = ("A python library to shorten urls using one of the url shortening services"),
    license = "MIT",
    keywords = "url shortening qrcode qr goo.gl bit.ly tinyurl.com j.mp bitly.com v.gd is.gd",
    platforms = ['Linux', 'Max OS X', 'Windows', 'BSD', 'Unix'],
    url = "https://github.com/fictivekin/pyshorturl",
    data_files=[
        ('.', ['README.rst']),
      ],
    packages = ['pyshorturl', 'pyshorturl/providers'],
    long_description = read('README.rst'),
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Other Audience",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Communications",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities",
    ],
    install_requires=[
        "requests"
    ],
)




import random

from .base import BaseShortener, ShortenerServiceError


GITIO_SERVICE_URL = 'https://git.io'


class GitioError(ShortenerServiceError):
    pass


class Gitio(BaseShortener):

    exception_class = GitioError
    service_url = GITIO_SERVICE_URL

    def __init__(self, api_key=None, logger=None):
        super().__init__(api_key, logger=logger)

    def _construct_request(self, long_url):  # pylint: disable=no-self-use
        """Construct the request body as multipart/form-data"""

        boundary = '-----------------------------' + str(int(random.random()*1e10))
        parts = []

        parts.append('--' + boundary)
        parts.append('Content-Disposition: form-data; name="url"')
        parts.append('')
        parts.append(str(long_url))

        parts.append('--' + boundary + '--')
        parts.append('')

        body = '\r\n'.join(parts)
        headers = {'content-type': 'multipart/form-data; boundary=' + boundary}

        return (headers, body)

    def shorten_url(self, long_url):
        request_url = self.service_url
        headers, body = self._construct_request(long_url)
        headers, response = self._do_http_request(request_url, data=body, headers=headers)  # pylint: disable=unused-variable

        short_url = headers.get('Location')

        if not short_url:
            raise self.exception_class('Unable to get short url for %s.' % long_url)

        return short_url




from urllib.parse import urlencode
from .base import BaseShortener, ShortenerServiceError


TINYURLCOM_SERVICE_URL = "http://tinyurl.com/api-create.php"


class TinyUrlcomError(ShortenerServiceError):
    pass


class TinyUrlcom(BaseShortener):

    exception_class = TinyUrlcomError
    service_url = TINYURLCOM_SERVICE_URL

    def __init__(self, logger=None):
        super().__init__(None, logger=logger)

    def _get_request_url(self):  # pylint: disable=no-self-use
        return self.service_url

    def shorten_url(self, long_url):
        data = {'url': long_url}
        data = urlencode(data)
        request_url = self._get_request_url()
        headers, response = self._do_http_request(request_url, data=data)  # pylint: disable=unused-variable

        if response == 'Error':
            raise self.exception_class('Received Error from tinyurl.com')

        return response
